"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChartTileLargeContainer = void 0;
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const ui_1 = require("@cs-bosch-medusa/ui");
const chart_tile_small_container_1 = require("./chart-tile-small-container");
const container_constants_1 = require("./container-constants");
const styles = react_native_1.StyleSheet.create({
    container: {
        flexDirection: "row",
        paddingHorizontal: ui_1.METRICS.CHART_TILE_CONTAINER.basePadding,
        paddingTop: ui_1.METRICS.CHART_TILE_CONTAINER.basePadding,
    },
});
const getContainerTileCount = (contentLimit, tiles) => {
    if (contentLimit === container_constants_1.DEFAULT_LARGE_CONTAINER_CONTENT_LIMIT) {
        return tiles.length > container_constants_1.DEFAULT_SMALL_CONTAINER_CONTENT_LIMIT + 1
            ? container_constants_1.DEFAULT_SMALL_CONTAINER_CONTENT_LIMIT
            : Math.round(tiles.length / 2);
    }
    return Math.min(tiles.length, 2);
};
const ChartTileLargeContainer = ({ children, activatedTiles, onChange, style, testID, contentLimit = container_constants_1.DEFAULT_LARGE_CONTAINER_CONTENT_LIMIT, }) => {
    const tiles = react_1.default.Children.toArray(children);
    /**
     * NOTE: align tiles to the left side in order to fill the left side first.
     */
    const tileCount = getContainerTileCount(contentLimit, tiles);
    /**
     * NOTE: In case the number of tiles is limited to 8, only the first two arrays will be filled,
     * as the container needs to be divided into 2 equal parts.
     * In case of 6 as maximum, all three arrays are used, so the container is divided into 3 parts.
     */
    const leftTiles = tiles.slice(0, tileCount);
    const middleTiles = tiles.slice(tileCount, 2 * tileCount);
    const rightTiles = tiles.slice(2 * tileCount);
    return (react_1.default.createElement(react_native_1.View, { style: [style, styles.container] },
        react_1.default.createElement(chart_tile_small_container_1.ChartTileSmallContainer, { activatedTiles: activatedTiles, contentLimit: contentLimit, onChange: onChange, testID: `${testID}.left` }, leftTiles),
        middleTiles.length > 0 && (react_1.default.createElement(chart_tile_small_container_1.ChartTileSmallContainer, { activatedTiles: activatedTiles, contentLimit: contentLimit, onChange: onChange, testID: `${testID}.middle` }, middleTiles)),
        rightTiles.length > 0 && (react_1.default.createElement(chart_tile_small_container_1.ChartTileSmallContainer, { activatedTiles: activatedTiles, contentLimit: contentLimit, onChange: onChange, testID: `${testID}.right` }, rightTiles))));
};
exports.ChartTileLargeContainer = ChartTileLargeContainer;
