import { FC } from "react";
import { Colors, TestProps } from "@cs-bosch-medusa/ui";
import type { ChartSelectionProps } from "./common-props";
import { ChartValue } from "./types";
export type BarChartProps = TestProps & {
    /** Width of the chart. */
    width: number;
    /** Height of the chart. */
    height: number;
    /** The number of ticks and labels to show below the X axis. */
    xTickCount: number;
    /** The number of lines and labels to show for the y axis. */
    yTickCount: number;
    /** The width in pixels of the bars. Defaults to `horizontalScale(6)`. */
    barWidth?: number;
    /** The list of parameters for which to draw bars. */
    activeParameters: string[];
    /**
     * A map containing all parameters and the corresponding data.
     * Each entry in `activeParameters` should have a corresponding key in this map.
     */
    values: Record<string, ChartValue[]>;
    /**
     * Function that formats the timestamps to be displayed on the X axis.
     *
     * @param date  the date to format
     */
    formatDate(date: Date): string;
    /**
     * Function that formats the values to be displayed on the Y axis.
     *
     * @param value  the value to format
     */
    formatValue(value: number): string;
    /** Parameter ids mapped to colors. */
    colors: Record<string, Colors>;
    /**
     * The time range inside which the values belong.
     * Format is [startTime, endTime].
     *
     * Timestamps should be given as unix timestamps (i.e. in seconds).
     */
    timestampRange: [number, number];
} & ChartSelectionProps;
export declare const BarChart: FC<BarChartProps>;
//# sourceMappingURL=bar-chart.d.ts.map