import { Colors, TestProps } from "@cs-bosch-medusa/ui";
import React, { FC } from "react";
import { StyleProp, ViewStyle } from "react-native";
export type HorizontalTimeLineProps = TestProps & {
    /**
     * The time range inside which the values belong.
     * Format is [startTime, endTime].
     */
    timestampRange: [startTime: number, endTime: number];
    /** The number of ticks to show below the X axis. */
    tickCount: number;
    /** The number of the small ticks to be displayed between ticks on the X axis. */
    smallTickCount: number;
    /**
     * Formatting function of the value to be displayed on the X axis.
     *
     * @param value the value of the tick on the X axis
     */
    formatLabel(value: number): string;
    /**
     * The frequency of the labels to show below the ticks.
     * It cannot be 0 or less.
     *
     * Default value is 2.
     */
    labelFrequency?: number;
    /**
     * Current timestamp to show the current time indicator.
     *
     * If this property is not given then the indicator is not shown.
     */
    currentTimestamp?: number;
    /**
     * Color of the current time indicator.
     *
     * Default color is the axis color.
     */
    indicatorColor?: Colors;
    /** Style of the container. */
    style?: StyleProp<ViewStyle>;
    /** Style of the children container. */
    containerStyle?: StyleProp<ViewStyle>;
    /**
     * Content of the time line container.
     *
     * For example it can be a `HorizontalStackedBar`.
     */
    children?: React.ReactNode;
};
/**
 * Horizontal Timeline container.
 */
export declare const HorizontalTimeLine: FC<HorizontalTimeLineProps>;
//# sourceMappingURL=horizontal-time-line.d.ts.map