"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CalendarListPicker = void 0;
const react_1 = __importStar(require("react"));
const moment_1 = __importDefault(require("moment"));
const react_native_1 = require("react-native");
const ui_1 = require("@cs-bosch-medusa/ui");
const calendar_list_item_1 = require("./calendar-list-item");
const calendar_ref_1 = require("./calendar-ref");
const styles = react_native_1.StyleSheet.create({
    listContainer: {
        flexGrow: 1,
    },
});
exports.CalendarListPicker = (0, react_1.forwardRef)(({ earliestDate, periodStart, resolution, maxDate, onChange, formatLabel, testID, }, ref) => {
    const calendarList = (0, react_1.useMemo)(() => {
        const endDate = (0, moment_1.default)(maxDate);
        return [
            ...generateCalendarItems(endDate, (0, moment_1.default)(earliestDate), resolution),
        ];
    }, [earliestDate, maxDate, resolution]);
    const calendarListRef = (0, calendar_ref_1.useCalendarListRef)(ref, calendarList, resolution, onChange);
    const initialScrollIndex = (0, react_1.useMemo)(() => {
        const currentItem = toCalendarItem((0, moment_1.default)(periodStart), resolution);
        return calendarList.findIndex((item) => item.key === currentItem.key);
        // the value is only relevant on the first render
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    const onItemPress = (0, react_1.useCallback)((newPeriodStart) => {
        onChange(newPeriodStart.toDate());
    }, [onChange]);
    const renderItem = (0, react_1.useCallback)(({ item }) => (react_1.default.createElement(calendar_list_item_1.CalendarListItem, { date: item.date, formatLabel: formatLabel, key: item.key, onPress: onItemPress, resolution: resolution, selected: (0, moment_1.default)(periodStart).isSame(item.date, resolution), testID: `${testID}.${item.key}` })), [formatLabel, onItemPress, periodStart, resolution, testID]);
    return (react_1.default.createElement(react_native_1.FlatList, { contentContainerStyle: styles.listContainer, data: calendarList, getItemLayout: calculateItemLayout, initialScrollIndex: initialScrollIndex, ref: calendarListRef, renderItem: renderItem, ...(0, ui_1.getTestIdProps)(testID) }));
});
function calculateItemLayout(_data, index) {
    return {
        length: calendar_list_item_1.LIST_ITEM_HEIGHT,
        offset: calendar_list_item_1.LIST_ITEM_HEIGHT * index,
        index,
    };
}
/**
 * Generates the list of items backwards from endDate (inclusive) to startDate (inclusive).
 * Yields objects that can be used in the data array.
 */
function* generateCalendarItems(endDate, startDate, resolution) {
    let period = endDate.clone();
    while (period.isAfter(startDate, resolution)) {
        yield toCalendarItem(period, resolution);
        period.subtract(1, resolution);
    }
    yield toCalendarItem(period, resolution);
}
/**
 * Converts a moment object to an item object to be used in the list.
 */
function toCalendarItem(date, resolution) {
    const periodStart = date.clone().startOf(resolution);
    return {
        key: periodStart.format("YYYY-MM"),
        date: periodStart,
    };
}
