"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnimatedPageIndicator = void 0;
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const react_native_svg_1 = require("react-native-svg");
const util_1 = require("../../util");
const theme_1 = require("../../theme");
const BASE_RADIUS = 3;
const ACTIVE_RADIUS = 5;
const SPACING = 12;
const PADDING = 5;
const HEIGHT = 2 * ACTIVE_RADIUS + 2 * PADDING;
const SPACED_CIRCLE_WIDTH = 2 * ACTIVE_RADIUS + SPACING;
const AnimatedCircle = react_native_1.Animated.createAnimatedComponent(react_native_svg_1.Circle);
const styles = react_native_1.StyleSheet.create({
    indicatorWrapper: {
        flexDirection: "row",
        justifyContent: "center",
        margin: theme_1.METRICS.CONTAINER.baseMargin,
    },
});
const AnimatedPageIndicator = ({ scrollX, parentWidth, pageCount, onPress, style, testID, }) => {
    const { theme } = (0, theme_1.useTheme)();
    const width = pageCount * SPACED_CIRCLE_WIDTH - SPACING + 2 * PADDING;
    return (react_1.default.createElement(react_native_1.View, { style: [styles.indicatorWrapper, style], ...(0, util_1.getTestIdProps)(testID) },
        react_1.default.createElement(react_native_svg_1.Svg, { height: HEIGHT, viewBox: `0 0 ${width} ${HEIGHT}`, width: width }, Array.from({ length: pageCount }).map((_, index) => {
            /*
            Set input range up in a way that it activates the current dot's animation
            only if the scroll position is within `parentWidth` distance of the current
            page's start point.

            E.g.:
            0   1   2   3   4
                 ^
            Would be within the range of indexes 1 and 2.
            */
            const inputRange = [
                (index - 1) * parentWidth,
                index * parentWidth,
                (index + 1) * parentWidth,
            ];
            const fillColor = scrollX.interpolate({
                inputRange,
                outputRange: [
                    theme.disabledColor,
                    theme.primaryColor,
                    theme.disabledColor,
                ],
                extrapolate: "clamp",
            });
            const radius = scrollX.interpolate({
                inputRange,
                outputRange: [BASE_RADIUS, ACTIVE_RADIUS, BASE_RADIUS],
                extrapolate: "clamp",
            });
            return (react_1.default.createElement(react_1.default.Fragment
            // eslint-disable-next-line react/no-array-index-key
            , { 
                // eslint-disable-next-line react/no-array-index-key
                key: `ind-circle-${index}` },
                react_1.default.createElement(AnimatedCircle, { cx: PADDING +
                        ACTIVE_RADIUS +
                        index * SPACED_CIRCLE_WIDTH, cy: HEIGHT / 2, fill: fillColor, r: radius }),
                react_1.default.createElement(PressableArea, { index: index, onPress: onPress })));
        }))));
};
exports.AnimatedPageIndicator = AnimatedPageIndicator;
const PressableArea = ({ index, onPress }) => {
    const onCirclePress = (0, react_1.useCallback)(() => {
        onPress === null || onPress === void 0 ? void 0 : onPress(index);
    }, [index, onPress]);
    if (!onPress) {
        // eslint-disable-next-line unicorn/no-null
        return null;
    }
    return (react_1.default.createElement(react_native_svg_1.Rect, { fill: "none", height: HEIGHT, onPress: onCirclePress, width: HEIGHT, x: index * SPACED_CIRCLE_WIDTH, y: 0 }));
};
