"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Dialog = exports.DialogVariant = void 0;
/* eslint-disable react/jsx-props-no-spreading */
const react_1 = __importStar(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const util_1 = require("../../util");
const button_1 = require("../button");
const icons_1 = require("../icons");
const text_1 = require("../text");
const containers_1 = require("../containers");
const styles = react_native_1.StyleSheet.create({
    buttonColumn: {
        flexDirection: "column",
        marginTop: theme_1.METRICS.CONTAINER.doubleBaseMargin,
    },
    buttonRow: {
        flexDirection: "row",
        justifyContent: "flex-end",
        marginTop: theme_1.METRICS.CONTAINER.doubleBaseMargin,
    },
    closeIcon: {
        alignItems: "flex-end",
        flexGrow: 1,
        height: "100%",
        paddingHorizontal: theme_1.METRICS.CONTAINER.basePadding,
    },
    dialogBody: {
        paddingBottom: theme_1.METRICS.CONTAINER.doubleBasePadding,
        width: "100%",
    },
    dialogContent: {
        flexGrow: 1,
        paddingHorizontal: theme_1.METRICS.CONTAINER.doubleBasePadding,
        paddingTop: theme_1.METRICS.CONTAINER.basePadding,
    },
    dialogShadow: {
        elevation: 7,
        shadowColor: theme_1.Colors.BLACK,
        shadowOffset: {
            width: 0,
            height: 3,
        },
        shadowOpacity: 0.4,
        shadowRadius: 4,
    },
    firstButtonVertical: {
        flex: 0,
    },
    flex: {
        flex: 1,
    },
    footerText: {
        marginTop: theme_1.METRICS.CONTAINER.baseMargin,
    },
    headerRow: {
        alignItems: "center",
        borderBottomWidth: 1,
        flexDirection: "row",
        paddingLeft: theme_1.METRICS.CONTAINER.doubleBasePadding,
        paddingVertical: theme_1.METRICS.CONTAINER.basePadding,
    },
    headerStrip: {
        borderTopWidth: theme_1.METRICS.CONTAINER.halfBaseMargin,
    },
    headline: {
        fontSize: theme_1.FontSize.LARGER_MEDIUM,
        marginBottom: theme_1.METRICS.CONTAINER.baseMargin,
    },
    iconContainer: {
        alignItems: "flex-start",
        height: "100%",
    },
    modalOverlay: {
        alignItems: "center",
        flex: 1,
        justifyContent: "center",
        paddingHorizontal: theme_1.METRICS.SCREEN.basePadding,
    },
    secondButtonHorizontal: {
        marginLeft: theme_1.METRICS.SCREEN.basePadding,
    },
    secondButtonVertical: {
        flex: 0,
        marginTop: theme_1.METRICS.SCREEN.basePadding,
    },
    smallButtons: {
        flex: 0,
        paddingHorizontal: theme_1.METRICS.CONTAINER.doubleBasePadding,
    },
    title: {
        flexShrink: 1,
        paddingRight: theme_1.METRICS.CONTAINER.doubleBasePadding,
    },
    titleWithIcon: {
        marginLeft: theme_1.METRICS.CONTAINER.halfBaseMargin,
    },
});
/** Specifies what type of dialog to render */
var DialogVariant;
(function (DialogVariant) {
    DialogVariant[DialogVariant["DEFAULT"] = 0] = "DEFAULT";
    DialogVariant[DialogVariant["INFO"] = 1] = "INFO";
    DialogVariant[DialogVariant["SUCCESS"] = 2] = "SUCCESS";
    DialogVariant[DialogVariant["WARNING"] = 3] = "WARNING";
    DialogVariant[DialogVariant["ERROR"] = 4] = "ERROR";
})(DialogVariant = exports.DialogVariant || (exports.DialogVariant = {}));
const Dialog = ({ visible = false, title, headline, icon: propsIcon, highlightColor: propsHighlightColor, description, footerText, firstButton, secondButton, smallButtons = false, verticalButtons = false, variant, inline = false, onClose, children, testID, style, withInput = false, }) => {
    const { theme } = (0, theme_1.useTheme)();
    const { icon, highlightColor } = useHeaderStyle({
        highlightColor: propsHighlightColor,
        icon: propsIcon,
        variant,
    });
    const firstButtonStyle = (0, react_1.useMemo)(() => react_native_1.StyleSheet.flatten([
        verticalButtons ? styles.firstButtonVertical : undefined,
        smallButtons ? styles.smallButtons : undefined,
    ]), [smallButtons, verticalButtons]);
    const secondButtonStyle = (0, react_1.useMemo)(() => react_native_1.StyleSheet.flatten([
        verticalButtons
            ? styles.secondButtonVertical
            : styles.secondButtonHorizontal,
        smallButtons ? styles.smallButtons : undefined,
    ]), [smallButtons, verticalButtons]);
    const dialogBody = (react_1.default.createElement(react_native_1.View, { style: react_native_1.StyleSheet.flatten([
            styles.modalOverlay,
            { backgroundColor: theme.modalBackgroundColor },
        ]) },
        react_1.default.createElement(react_native_1.View, { style: react_native_1.StyleSheet.flatten([
                styles.dialogBody,
                styles.dialogShadow,
                { backgroundColor: theme.backgroundColor },
                style,
            ]), ...(0, util_1.getTestIdProps)(testID) },
            title ? (react_1.default.createElement(react_native_1.View, { style: [
                    styles.headerRow,
                    { borderBottomColor: theme.separatorColor },
                    ...(highlightColor
                        ? [
                            styles.headerStrip,
                            {
                                borderTopColor: highlightColor,
                            },
                        ]
                        : []),
                ] },
                react_1.default.createElement(react_native_1.View, { style: styles.iconContainer }, icon),
                react_1.default.createElement(text_1.Title, { style: react_native_1.StyleSheet.flatten([
                        styles.title,
                        icon ? styles.titleWithIcon : undefined,
                    ]) }, title),
                onClose && (react_1.default.createElement(react_native_1.TouchableOpacity, { activeOpacity: theme.activeOpacity, onPress: onClose, style: styles.closeIcon, ...(0, util_1.getTestIdProps)(`${testID}.close`) },
                    react_1.default.createElement(icons_1.CloseIcon, { iconSize: icons_1.IconSize.XSMALL }))))) : // eslint-disable-next-line unicorn/no-null
                null,
            react_1.default.createElement(react_native_1.View, { style: styles.dialogContent },
                headline ? (react_1.default.createElement(text_1.Title, { style: styles.headline }, headline)) : undefined,
                description ? react_1.default.createElement(text_1.Label, null, description) : children,
                footerText ? (react_1.default.createElement(text_1.Subtitle, { style: styles.footerText }, footerText)) : undefined,
                (firstButton || secondButton) && (react_1.default.createElement(react_native_1.View, { style: verticalButtons
                        ? styles.buttonColumn
                        : styles.buttonRow },
                    firstButton && (react_1.default.createElement(button_1.Button, { disabled: firstButton.disabled, onPress: firstButton.onPress, style: firstButtonStyle, testID: firstButton.testID, text: firstButton.text, variant: firstButton.variant })),
                    secondButton && (react_1.default.createElement(button_1.Button, { disabled: secondButton.disabled, onPress: secondButton.onPress, style: secondButtonStyle, testID: secondButton.testID, text: secondButton.text, variant: secondButton.variant }))))))));
    return !inline ? (react_1.default.createElement(react_native_1.Modal, { onRequestClose: onClose, transparent: true, visible: visible, ...(0, util_1.getTestIdProps)(`${testID}.modal`) }, withInput ? (react_1.default.createElement(containers_1.KeyboardContainer, null, dialogBody)) : (dialogBody))) : (react_1.default.createElement(react_native_1.View, { style: styles.flex }, dialogBody));
};
exports.Dialog = Dialog;
const useHeaderStyle = ({ highlightColor, icon, variant = DialogVariant.DEFAULT, }) => {
    const { theme } = (0, theme_1.useTheme)();
    switch (variant) {
        case DialogVariant.DEFAULT:
            return {
                icon,
                highlightColor,
            };
        case DialogVariant.INFO: {
            const color = highlightColor !== null && highlightColor !== void 0 ? highlightColor : theme.info.lineColor;
            return {
                icon: icon !== null && icon !== void 0 ? icon : (react_1.default.createElement(icons_1.AlertInfoFilledIcon, { fillColor: color, iconSize: icons_1.IconSize.XSMALL })),
                highlightColor: color,
            };
        }
        case DialogVariant.SUCCESS: {
            const color = highlightColor !== null && highlightColor !== void 0 ? highlightColor : theme.success.lineColor;
            return {
                icon: icon !== null && icon !== void 0 ? icon : (react_1.default.createElement(icons_1.AlertSuccessFilledIcon, { fillColor: color, iconSize: icons_1.IconSize.XSMALL })),
                highlightColor: color,
            };
        }
        case DialogVariant.WARNING: {
            const color = highlightColor !== null && highlightColor !== void 0 ? highlightColor : theme.warning.lineColor;
            return {
                icon: icon !== null && icon !== void 0 ? icon : (react_1.default.createElement(icons_1.AlertWarningTwoToneIcon, { fillColor: color, iconSize: icons_1.IconSize.XSMALL, secondToneColor: theme_1.Colors.BLACK })),
                highlightColor: color,
            };
        }
        case DialogVariant.ERROR: {
            const color = highlightColor !== null && highlightColor !== void 0 ? highlightColor : theme.error.lineColor;
            return {
                icon: icon !== null && icon !== void 0 ? icon : (react_1.default.createElement(icons_1.AlertErrorFilledIcon, { fillColor: color, iconSize: icons_1.IconSize.XSMALL })),
                highlightColor: color,
            };
        }
    }
};
