"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Dropdown = void 0;
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const dropdown_item_1 = require("./dropdown-item");
const theme_1 = require("../../theme");
const text_1 = require("../text");
const custom_dropdown_1 = require("./custom-dropdown");
const styles = react_native_1.StyleSheet.create({
    dropdownHeaderContent: {
        flex: 1,
        justifyContent: "center",
    },
    valueText: {
        minHeight: theme_1.FontSize.MEDIUM + theme_1.METRICS.CONTAINER.halfBasePadding,
    },
});
const Dropdown = ({ value, label, placeholder, items, disabled = false, onOptionPress, testID, screenInFocus, }) => {
    const { theme } = (0, theme_1.useTheme)();
    const valueText = value ? items[value].label : undefined;
    const textColor = disabled
        ? theme.dropdown.disabledColor
        : valueText
            ? theme.textColor
            : theme.inputs.placeholderColor;
    const labelColor = disabled
        ? theme.dropdown.disabledColor
        : theme.textColor;
    return (react_1.default.createElement(custom_dropdown_1.CustomDropdown, { disabled: disabled, headerComponent: react_1.default.createElement(react_native_1.View, { style: styles.dropdownHeaderContent },
            label && (react_1.default.createElement(text_1.Subtitle, { ellipsis: true, style: {
                    color: labelColor,
                    fontSize: theme_1.FontSize.TINY,
                } }, label)),
            react_1.default.createElement(text_1.Label, { ellipsis: true, style: [
                    styles.valueText,
                    {
                        color: textColor,
                    },
                ] }, valueText !== null && valueText !== void 0 ? valueText : placeholder)), onOptionPress: onOptionPress, screenInFocus: screenInFocus, testID: testID, value: value }, items &&
        Object.entries(items).map(([optionValue, item]) => (react_1.default.createElement(dropdown_item_1.DropdownItem, { disabled: item.disabled, key: optionValue, label: item.label, testID: `${testID}.options.${optionValue}`, value: optionValue })))));
};
exports.Dropdown = Dropdown;
