import React from "react";
import { CommonValidationProps } from "./text-field-validation-info";
import { InputFieldProps } from "./input-field";
import { TestProps } from "../../util";
import { TextFieldRef } from "./text-field-ref";
export type NumberInputFieldProps = Pick<InputFieldProps, "disabled" | "onFocus" | "onBlur" | "textAlign"> & CommonValidationProps & TestProps & {
    /** Current value of the text field. */
    value: number;
    /**
     * Precision of `value`. The number is the amount of decimal digits allowed after
     * the decimal separator. In case the entered number in the field will exceed the number
     * of digits specified in `precision`, the excess digits will be truncated.
     *
     * This property should be in sync with `step`, e.g: `step = 0.1 => precision = 1`.
     * The `precision` must be *greater* or *equal* to the number of decimal digits after
     * the decimal separator of `step`.
     *
     * Also take care to account for the precision of the `min` and `max` properties.
     */
    precision?: number;
    /**
     * It will increment or decrement the value by pressing on the icons within the
     * text field by the amount specified.
     */
    step?: number;
    /**
     * The minimum allowed value. If undefined, no checking will be performed.
     * If minimum is reached, the `-` button is disabled.
     */
    min?: number;
    /**
     * The maximum allowed value. If undefined, no checking will be performed.
     * If maximum is reached, the `+` button is disabled.
     */
    max?: number;
    /** Label to display above the text field. */
    label?: string;
    /**
     * Locale based decimal separator which will be displayed within the input field
     * in case of a decimal.
     */
    decimalSeparator?: "." | ",";
    /**
     * Whether to show + and - buttons on input.
     * Defaults to `true`.
     */
    showButtons?: boolean;
    /**
     * Callback that is called when the value in the text field changes.
     * It will be called on `onBlur` in case the value is not a number
     * (e.g.: empty string) or it ends in a decimal separator.
     */
    onValueChange(value: number): void;
};
export declare const NumberInputField: React.ForwardRefExoticComponent<Pick<InputFieldProps, "disabled" | "onBlur" | "onFocus" | "textAlign"> & CommonValidationProps & TestProps & {
    /** Current value of the text field. */
    value: number;
    /**
     * Precision of `value`. The number is the amount of decimal digits allowed after
     * the decimal separator. In case the entered number in the field will exceed the number
     * of digits specified in `precision`, the excess digits will be truncated.
     *
     * This property should be in sync with `step`, e.g: `step = 0.1 => precision = 1`.
     * The `precision` must be *greater* or *equal* to the number of decimal digits after
     * the decimal separator of `step`.
     *
     * Also take care to account for the precision of the `min` and `max` properties.
     */
    precision?: number | undefined;
    /**
     * It will increment or decrement the value by pressing on the icons within the
     * text field by the amount specified.
     */
    step?: number | undefined;
    /**
     * The minimum allowed value. If undefined, no checking will be performed.
     * If minimum is reached, the `-` button is disabled.
     */
    min?: number | undefined;
    /**
     * The maximum allowed value. If undefined, no checking will be performed.
     * If maximum is reached, the `+` button is disabled.
     */
    max?: number | undefined;
    /** Label to display above the text field. */
    label?: string | undefined;
    /**
     * Locale based decimal separator which will be displayed within the input field
     * in case of a decimal.
     */
    decimalSeparator?: "." | "," | undefined;
    /**
     * Whether to show + and - buttons on input.
     * Defaults to `true`.
     */
    showButtons?: boolean | undefined;
    /**
     * Callback that is called when the value in the text field changes.
     * It will be called on `onBlur` in case the value is not a number
     * (e.g.: empty string) or it ends in a decimal separator.
     */
    onValueChange(value: number): void;
} & React.RefAttributes<TextFieldRef>>;
//# sourceMappingURL=number-input-field.d.ts.map