"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProcessIndicator = void 0;
const lodash_1 = require("lodash");
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const react_native_svg_1 = require("react-native-svg");
const theme_1 = require("../../theme");
const icons_1 = require("../icons");
const util_1 = require("../../util");
const SIZES = {
    radius: (0, theme_1.verticalScale)(14.5),
    stroke: 1.5,
    lineStroke: 1,
    lineWidth: (0, theme_1.horizontalScale)(21),
    shortLineWidth: (0, theme_1.horizontalScale)(14),
    fontSize: (0, theme_1.verticalScale)(15),
    iconSize: (0, theme_1.verticalScale)(21),
    iconExtraThickness: 6,
};
const CIRCLE_SIZE = (SIZES.radius + SIZES.stroke) * 2;
const SCREEN_WIDTH = react_native_1.Dimensions.get("window").width;
const styles = react_native_1.StyleSheet.create({
    indicatorWrapper: {
        flexDirection: "row",
        justifyContent: "center",
        padding: theme_1.METRICS.SCREEN.basePadding,
    },
});
var StageType;
(function (StageType) {
    StageType[StageType["COMPLETED"] = 0] = "COMPLETED";
    StageType[StageType["CURRENT"] = 1] = "CURRENT";
    StageType[StageType["REMAINING"] = 2] = "REMAINING";
})(StageType || (StageType = {}));
const buildStageArray = (currentStage, maxStages) => {
    const completed = [...(0, lodash_1.range)(1, currentStage, 1)].map((i) => ({
        number: i,
        type: StageType.COMPLETED,
    }));
    const current = currentStage <= maxStages
        ? [{ number: currentStage, type: StageType.CURRENT }]
        : [];
    const remaining = [...(0, lodash_1.range)(currentStage + 1, maxStages + 1, 1)].map((i) => ({
        number: i,
        type: StageType.REMAINING,
    }));
    return [...completed, ...current, ...remaining];
};
const renderStage = (stage, index, lineWidth, theme) => {
    const strokeColor = stage.type === StageType.REMAINING
        ? theme.disabledColor
        : theme.primaryColor;
    const backgroundColor = stage.type === StageType.COMPLETED
        ? theme.primaryColor
        : theme.backgroundColor;
    const prevCount = Math.max(0, index - 1);
    return (react_1.default.createElement(react_1.default.Fragment, { key: index },
        react_1.default.createElement(react_native_svg_1.G, { x: index * CIRCLE_SIZE + prevCount * lineWidth },
            index > 0 && (react_1.default.createElement(react_native_svg_1.Line, { stroke: strokeColor, strokeWidth: SIZES.lineStroke, x1: SIZES.stroke, x2: lineWidth - SIZES.stroke, y: SIZES.radius + SIZES.stroke })),
            react_1.default.createElement(react_native_svg_1.G, { x: index > 0 ? lineWidth : 0 },
                react_1.default.createElement(react_native_svg_1.Circle, { cx: SIZES.radius + SIZES.stroke, cy: SIZES.radius + SIZES.stroke, fill: backgroundColor, r: SIZES.radius, stroke: strokeColor, strokeWidth: SIZES.stroke }),
                stage.type !== StageType.COMPLETED && (react_1.default.createElement(react_native_svg_1.Text, { alignmentBaseline: "central", fill: strokeColor, fontFamily: theme.regularFont, fontSize: SIZES.fontSize, fontWeight: 500, textAnchor: "middle", x: CIRCLE_SIZE / 2, y: CIRCLE_SIZE / 2 }, stage.number)),
                stage.type === StageType.COMPLETED && (react_1.default.createElement(react_native_svg_1.G, { x: (CIRCLE_SIZE - SIZES.iconSize) / 2, y: (CIRCLE_SIZE - SIZES.iconSize) / 2 },
                    react_1.default.createElement(icons_1.CheckmarkIcon, { extraThickness: SIZES.iconExtraThickness, fillColor: theme.backgroundColor, iconSizeInPixels: SIZES.iconSize })))))));
};
const ProcessIndicator = ({ maxStages, maxWidth = SCREEN_WIDTH - theme_1.METRICS.SCREEN.basePadding * 2, currentStage, shortLines = false, testID, }) => {
    const { theme } = (0, theme_1.useTheme)();
    const stages = buildStageArray(currentStage, maxStages);
    const lineWidth = shortLines ? SIZES.shortLineWidth : SIZES.lineWidth;
    const width = CIRCLE_SIZE * maxStages + lineWidth * (maxStages - 1);
    const svgWidth = Math.min(width, maxWidth);
    return (
    // eslint-disable-next-line react/jsx-props-no-spreading
    react_1.default.createElement(react_native_1.View, { style: styles.indicatorWrapper, ...(0, util_1.getTestIdProps)(testID) },
        react_1.default.createElement(react_native_svg_1.Svg, { height: CIRCLE_SIZE, viewBox: `0 0 ${width} ${CIRCLE_SIZE}`, width: svgWidth }, stages.map((stage, index) => renderStage(stage, index, lineWidth, theme)))));
};
exports.ProcessIndicator = ProcessIndicator;
