"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProgressBar = void 0;
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const progress_bar_wrapper_1 = require("./progress-bar-wrapper");
const styles = react_native_1.StyleSheet.create({
    progressBar: {
        height: "100%",
    },
    progressBarWrapper: {
        paddingHorizontal: theme_1.METRICS.CONTAINER.basePadding,
    },
});
const ProgressBar = ({ value, min = 0, max = 100, testID, }) => {
    const { theme } = (0, theme_1.useTheme)();
    const progressPercent = ((value - min) / (max - min)) * 100;
    return (
    /**
     * Horizontal padding is passed for the wrapper around the progress bar component, so it behaves
     * the same way it did before the wrapper was implemented. This is needed because in the apps where it got
     * introduced already, the determinate progress bar is displayed with the horizontal padding being considered,
     * even though according to the Bosch brand guide it has only vertical paddings.
     */
    react_1.default.createElement(progress_bar_wrapper_1.ProgressBarWrapper, { style: styles.progressBarWrapper, testID: testID },
        react_1.default.createElement(react_native_1.View, { style: [
                styles.progressBar,
                {
                    width: `${progressPercent}%`,
                    backgroundColor: theme.primaryColor,
                },
            ] })));
};
exports.ProgressBar = ProgressBar;
