import React from "react";
import { StyleProp, ViewStyle } from "react-native";
import { TestProps } from "../../util";
export type RadioButtonProps<T> = TestProps & {
    /**
     * The label to display next to the radio button.
     * If the label is not provided, the children will be rendered.
     */
    label?: string;
    /** The value the button represents. */
    value: T;
    /**
     * Whether to show separator below the radio button.
     * Defaults to `false`.
     */
    separator?: boolean;
    /** Additional styling to apply to button's row. */
    style?: StyleProp<ViewStyle>;
};
/**
 * Internal interface with properties that are overwritten by RadioButtonContainer.
 * They would have no effect if specified from the outside, so hide them from the public
 * interface.
 */
export type InternalRadioButtonProps<T> = RadioButtonProps<T> & {
    selected: boolean;
    pressed: boolean;
    disabled: boolean;
};
/**
 * Component to be used inside RadioButtonContainer to list the available options.
 */
export declare const RadioButton: <T>(props: React.PropsWithChildren<RadioButtonProps<T>>) => React.JSX.Element;
//# sourceMappingURL=radio-button.d.ts.map