"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SegmentedControlTab = void 0;
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const text_1 = require("../text");
const theme_1 = require("../../theme");
const styles = react_native_1.StyleSheet.create({
    tab: {
        alignItems: "center",
        borderBottomWidth: 1,
        borderRightWidth: 1,
        borderTopWidth: 1,
        flexDirection: "row",
        flexGrow: 1,
        justifyContent: "center",
        padding: theme_1.METRICS.CONTAINER.basePadding,
    },
});
/**
 * Component to be used inside SegmentedControl to list the available options.
 */
function SegmentedControlTab(props) {
    const { theme } = (0, theme_1.useTheme)();
    const { text, icon: Icon, selected, disabled, first, pressed, } = props;
    const colorGroup = selected
        ? theme.segmentedControl.selected
        : theme.segmentedControl.tab;
    let colors = colorGroup.regular;
    if (disabled) {
        colors = colorGroup.disabled;
    }
    else if (pressed) {
        colors = colorGroup.pressed;
    }
    const { backgroundColor, contentColor, borderColor } = colors;
    const borderLeftWidth = first ? 1 : 0;
    return (react_1.default.createElement(react_native_1.View, { style: [
            styles.tab,
            {
                backgroundColor,
                borderColor,
                borderLeftWidth,
            },
        ] },
        Icon && (react_1.default.createElement(Icon, { fillColor: contentColor, iconSize: theme.segmentedControl.iconSize })),
        text !== undefined && (react_1.default.createElement(text_1.Label, { style: {
                color: contentColor,
                fontSize: theme.segmentedControl.fontSize,
            } }, text))));
}
exports.SegmentedControlTab = SegmentedControlTab;
