import React from "react";
import { TestProps } from "../../util";
export type SegmentedControlProps<T> = TestProps & {
    /** The value that is currently selected. */
    value: T;
    /** Multiple `SegmentedControlTab` components passed as children. */
    children: React.ReactNode;
    /**
     * If set to `false` each tab will only be as large as its content needs it.
     *
     * By default, the segmented control takes up all available horizontal space
     * and tab widths are distributed equally so that each tab has the same
     * width.
     */
    stretch?: boolean;
    /** Disable interactions with the component. */
    disabled?: boolean;
    /**
     * Callback to be called when a tab is pressed.
     *
     * @param newValue the value of the pressed tab
     */
    onChange(newValue: T): void;
};
/**
 * A component that lets the user select an option from the list of possible
 * values. Use when there are few values (2-4). The options are shown besides
 * each other and are visually similar to buttons.
 */
export declare function SegmentedControl<T>(props: React.PropsWithChildren<SegmentedControlProps<T>>): React.ReactElement | null;
//# sourceMappingURL=segmented-control.d.ts.map