"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tile = void 0;
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const text_1 = require("../text");
const util_1 = require("../../util");
const styles = react_native_1.StyleSheet.create({
    container: {
        flex: 1,
        justifyContent: "center",
        padding: theme_1.METRICS.CONTAINER.basePadding,
    },
    contentView: {
        alignItems: "center",
        justifyContent: "center",
    },
    iconContainer: {
        marginBottom: theme_1.METRICS.CONTAINER.doubleBaseMargin,
    },
    squareView: {
        aspectRatio: 1,
        flex: 1,
    },
    text: {
        textAlign: "center",
    },
});
/**
 * A square tile to be used as a navigation element or button, mostly on dashboards.
 */
const Tile = (props) => {
    const { theme } = (0, theme_1.useTheme)();
    const { text, icon: Icon, onPress, disabled = false, style, color, testID, } = props;
    const backgroundColor = disabled
        ? theme.disabledColor
        : color !== null && color !== void 0 ? color : theme.primaryColor;
    const tileStyle = react_native_1.StyleSheet.flatten([
        styles.container,
        { backgroundColor },
    ]);
    const textColor = disabled ? theme.disabledTextColor : theme.tileTextColor;
    const labelStyle = react_native_1.StyleSheet.flatten([styles.text, { color: textColor }]);
    return (react_1.default.createElement(react_native_1.TouchableOpacity, { activeOpacity: 0.8, disabled: disabled, onPress: onPress, style: [styles.squareView, style], ...(0, util_1.getTestIdProps)(testID) },
        react_1.default.createElement(react_native_1.View, { style: theme_1.APP_STYLES.flex },
            react_1.default.createElement(react_native_1.View, { style: tileStyle },
                react_1.default.createElement(react_native_1.View, { style: styles.contentView },
                    Icon && (react_1.default.createElement(react_native_1.View, { style: styles.iconContainer },
                        react_1.default.createElement(Icon, { fillColor: textColor, iconSizeInPixels: (0, theme_1.verticalScale)(40) }))),
                    react_1.default.createElement(text_1.Label, { style: labelStyle }, text))))));
};
exports.Tile = Tile;
