"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WhatsNewModal = void 0;
const react_1 = __importDefault(require("react"));
const react_native_1 = require("react-native");
const theme_1 = require("../../theme");
const util_1 = require("../../util");
const button_1 = require("../button");
const containers_1 = require("../containers");
const icons_1 = require("../icons");
const text_1 = require("../text");
const scrollable_news_section_1 = require("./scrollable-news-section");
const styles = react_native_1.StyleSheet.create({
    centerContent: {
        flex: 1,
        justifyContent: "center",
        paddingBottom: theme_1.METRICS.CONTAINER.doubleBasePadding,
    },
    closeIcon: {
        alignSelf: "flex-end",
    },
    title: {
        fontSize: theme_1.FontSize.EXTRA_LARGE,
        paddingBottom: theme_1.METRICS.CONTAINER.basePadding,
        textAlign: "center",
    },
});
const WhatsNewModal = ({ visible, onClose, title, newsSections, buttonText, testID, }) => {
    const { theme } = (0, theme_1.useTheme)();
    if (!visible) {
        // eslint-disable-next-line unicorn/no-null
        return null;
    }
    return (react_1.default.createElement(react_native_1.Modal, { onRequestClose: onClose, transparent: true, visible: true },
        react_1.default.createElement(containers_1.Container, { style: [
                theme_1.MODAL_STYLES.modalView,
                { backgroundColor: theme.whatsNew.backgroundColor },
            ], testID: testID },
            react_1.default.createElement(react_native_1.SafeAreaView, { style: theme_1.APP_STYLES.flex },
                react_1.default.createElement(react_native_1.TouchableOpacity
                // eslint-disable-next-line react/jsx-props-no-spreading
                , { ...(0, util_1.getTestIdProps)(`${testID}.closeIcon`), onPress: onClose, style: styles.closeIcon },
                    react_1.default.createElement(icons_1.CloseIcon, { fillColor: theme.whatsNew.textColor, iconSize: icons_1.IconSize.XSMALL })),
                react_1.default.createElement(react_native_1.View, { style: styles.centerContent },
                    react_1.default.createElement(text_1.Label, { style: [
                            styles.title,
                            {
                                color: theme.whatsNew.textColor,
                                fontFamily: theme.boldFont,
                            },
                        ] }, title),
                    react_1.default.createElement(scrollable_news_section_1.ScrollableNewSection, { newsSections: newsSections })),
                react_1.default.createElement(theme_1.ThemeProvider, { initialTheme: {
                        ...theme,
                        backgroundColor: theme_1.Colors.TRANSPARENT,
                        primaryColor: theme.whatsNew.textColor,
                        pressedColor: theme.whatsNew.pressedColor,
                        secondaryPressedColor: theme.whatsNew.secondaryPressedColor,
                    } },
                    react_1.default.createElement(button_1.Button, { onPress: onClose, testID: `${testID}.button`, text: buttonText, variant: button_1.ButtonVariant.SECONDARY }))))));
};
exports.WhatsNewModal = WhatsNewModal;
