package com.bosch.tt.privacypolicy.rn.assembler;

import com.bosch.tt.privacypolicy.core.MetaActivityDocument;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;

import java.util.ArrayList;
import java.util.List;

public class ActivityConsentAssembler implements Assembler<MetaActivityDocument> {

    @Override
    public WritableArray createWritableDTOArray(List<MetaActivityDocument> models) {
        final WritableArray writableArray = Arguments.createArray();
        for (MetaActivityDocument document : models) {
            writableArray.pushMap(createWritableDTO(document));
        }
        return writableArray;
    }

    @Override
    public WritableMap createWritableDTO(MetaActivityDocument model) {
        final WritableMap documentDTO = Arguments.createMap();
        documentDTO.putString("activityId", model.getId());
        documentDTO.putInt("revision", model.getRevision());
        documentDTO.putString("name", model.getName());
        documentDTO.putString("country", model.getCountry());
        documentDTO.putString("language", model.getLanguage());
        documentDTO.putString("validFrom", model.getValidFrom());
        documentDTO.putString("timestamp", model.getTimestamp());
        return documentDTO;
    }

    @Override
    public MetaActivityDocument createModel(ReadableMap readableMap) {
        return new MetaActivityDocument(
                readableMap.getString("activityId"),
                readableMap.getInt("revision"),
                readableMap.getString("name"),
                readableMap.getString("country"),
                readableMap.getString("language"),
                readableMap.getString("validFrom"),
                readableMap.getString("timestamp")
        );
    }

    @Override
    public List createModelArray(ReadableArray readableArray) {
        final List<MetaActivityDocument> documents = new ArrayList<>();
        for (int i = 0; i < readableArray.size(); ++i) {
            documents.add(createModel(readableArray.getMap(i)));
        }
        return documents;
    }
}
