package com.bosch.tt.privacypolicy.rn.assembler;

import com.bosch.tt.privacypolicy.core.MetaDocument;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;

import java.util.ArrayList;
import java.util.List;

/**
 * Assembler class for the MetaDocument model
 */
public class DocumentAssembler implements Assembler<MetaDocument> {

    @Override
    public WritableMap createWritableDTO(MetaDocument model) {
        final WritableMap documentDTO = Arguments.createMap();
        if (model != null) {
            documentDTO.putString("id", model.getId());
            documentDTO.putString("country", model.getCountry());
            documentDTO.putString("language", model.getLanguage());
            documentDTO.putString("validFrom", model.getValidFrom());
            documentDTO.putString("createdOn", model.getCreatedOn());
        }
        return documentDTO;
    }

    @Override
    public MetaDocument createModel(ReadableMap readableMap) {
        return new MetaDocument(
            readableMap.getString("id"),
            readableMap.getString("country"),
            readableMap.getString("language"),
            readableMap.getString("validFrom"),
            readableMap.getString("createdOn")
        );
    }

    @Override
    public WritableArray createWritableDTOArray(List<MetaDocument> models) {
        final WritableArray writableArray = Arguments.createArray();
        for (MetaDocument document : models) {
            writableArray.pushMap(createWritableDTO(document));
        }
        return writableArray;
    }

    @Override
    public List<MetaDocument> createModelArray(ReadableArray readableArray) {
        final List<MetaDocument> documents = new ArrayList<>();
        for (int i = 0; i < readableArray.size(); ++i) {
            documents.add(createModel(readableArray.getMap(i)));
        }
        return documents;
    }
}
