package com.bosch.tt.privacypolicy.rn.provider;

import android.content.Context;
import android.content.res.AssetManager;
import android.util.Log;

import com.bosch.tt.privacypolicy.valueprovider.Response;
import com.bosch.tt.privacypolicy.valueprovider.ValueProvider;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

public abstract class OfflineValueProvider implements ValueProvider {

    private static final String TAG = OfflineValueProvider.class.getSimpleName();

    private Context context;

    public OfflineValueProvider(final Context context) {
        this.context = context;
    }

    @Override
    public Response getDocument(String code, String language) {
        String fileName = getFolderName() + "/" + getFilePrefix() + "_" + code.toLowerCase() + "_" + language.toLowerCase() + ".json";
        JSONObject documentJSON = this.jsonObjectFromFile(context, fileName);
        if (documentJSON != null) {
            return new Response(true, documentJSON.toString(), null);
        } else {
            return new Response(false, null, "The json reading failed!");
        }
    }

    @Override
    public Response getAvailableDocuments() {
        AssetManager assetManager = context.getResources().getAssets();
        try {
            String fileNames[] = assetManager.list(getFolderName());
            JSONArray metaDocumentJSON = new JSONArray("[]");
            if (fileNames != null) {
                for (String fileName : fileNames) {
                    JSONObject jsonObject = this.jsonObjectFromFile(context, getFolderName() + "/" + fileName);
                    if (jsonObject != null) {
                        metaDocumentJSON.put(jsonObject);
                    }
                }
            }
            return new Response(true, metaDocumentJSON.toString(), null);
        } catch (IOException | JSONException e) {
            Log.e(TAG, "Error while retrieving the available documents!", e);
            return new Response(false, null, "The json reading failed!");
        }
    }

    private JSONObject jsonObjectFromFile(Context context, String fileName) {
        StringBuilder jsonStringBuilder = new StringBuilder();
        InputStream fIn = null;
        InputStreamReader isr = null;
        BufferedReader input = null;
        try {
            fIn = context.getResources().getAssets().open(fileName, Context.MODE_WORLD_READABLE);
            isr = new InputStreamReader(fIn);
            input = new BufferedReader(isr);
            String line;
            while ((line = input.readLine()) != null) {
                jsonStringBuilder.append(line);
            }
        } catch (Exception e) {
            Log.e(TAG, "Error in file reading!", e);
        } finally {
            try {
                if (isr != null) {
                    isr.close();
                }
                if (fIn != null) {
                    fIn.close();
                }
                if (input != null) {
                    input.close();
                }
            } catch (Exception e2) {
                Log.e(TAG, "Error in stream closing", e2);
            }
        }
        try {
            return new JSONObject(jsonStringBuilder.toString());
        } catch (JSONException e) {
            Log.e(TAG, "Error in json conversion", e);
        }
        return null;
    }

    protected abstract String getFolderName();

    protected abstract String getFilePrefix();
}
