package com.bosch.tt.privacypolicy.rn.ui;

import android.view.View;

import com.bosch.tt.privacypolicy.ui.TTPrivacyComponent;
import com.facebook.react.uimanager.ThemedReactContext;
import com.facebook.react.uimanager.events.RCTEventEmitter;

/**
 * This child class is necessary, because when an update occurs on the custom view which will affect the view's size, the update will not be visible.
 * Should override the "requestLayout" method and set the new layout according to changes.
 *
 * See issues:
 *  https://github.com/facebook/react-native/issues/4990
 *  https://stackoverflow.com/questions/39836356/react-native-resize-custom-ui-component
 */
public class ReactTTPrivacyComponent extends TTPrivacyComponent {

    private final ThemedReactContext reactContext;

    private final Runnable measureAndLayout = new Runnable() {
        @Override
        public void run() {
            measure(
                MeasureSpec.makeMeasureSpec(getWidth(), MeasureSpec.EXACTLY),
                MeasureSpec.makeMeasureSpec(getHeight(), MeasureSpec.EXACTLY)
            );
            layout(getLeft(), getTop(), getRight(), getBottom());
        }
    };

    public ReactTTPrivacyComponent(ThemedReactContext context) {
        super(context);
        this.reactContext = context;
    }

    @Override
    public void requestLayout() {
        super.requestLayout();
        post(measureAndLayout);
    }

    @Override
    public void onScrollChanged(int x, int y, int oldX, int oldY) {
        super.onScrollChanged(x, y, oldX, oldY);
        View view = getChildAt(getChildCount() - 1);
        int diff = (view.getBottom() - (getHeight() + getScrollY()));
        if(diff == 0) {
            this.reactContext.getJSModule(RCTEventEmitter.class).receiveEvent(this.getId(), TTComponentEvent.ON_SCROLL_ENDED.toString(), null);
        }
    }
}
