//
//  ProviderManager.m
//  RNPrivacyComponent
//
//  Created by István Bege on 31/10/2018.
//  Copyright (C) 2018, Bosch Thermotechnik GmbH. All rights reserved.
//
#import <Foundation/Foundation.h>

#import <TTPrivacyComponent/TTPPrivacyProviderBuilder.h>
#import <TTPrivacyComponent/TTPActivityProviderBuilder.h>

#import "ProviderManager.h"
#import "PrivacyPolicyOfflineProvider.h"
#import "TermsOfUseProvider.h"
#import "ActivityConsentOfflineProvider.h"

static ProviderManager *privacyManager = nil;

@implementation ProviderManager

@synthesize selectedCountry, supportedCountries, privacyProvider, termsOfUseProvider, activityProvider;

- (void)initPrivacyProviderWithLeanIX:(NSString*)leanIX productionMode:(BOOL)productionMode {
    TTPPrivacyProviderBuilder* builder = [[TTPPrivacyProviderBuilder alloc] initWithLeanIX:leanIX offlineProvider:[[PrivacyPolicyOfflineProvider alloc] init]];
    if (!productionMode) {
        [builder onlineProvider:[[TTPPreviewValueProvider alloc] initWithLeanIX:leanIX]];
    }
    privacyProvider = [builder build];
}

- (id) init {
    self = [super init];
    return self;
}

+ (instancetype)getInstance
{
    static dispatch_once_t onceToken;

    dispatch_once(&onceToken, ^{
        privacyManager = [[self alloc] init];
    });
    return privacyManager;
}

- (void)initTermsOfUseProvider {
    TermsOfUseProvider* valueProvider = [[TermsOfUseProvider alloc] init];
    TTPPrivacyProviderBuilder* builder = [[TTPPrivacyProviderBuilder alloc] initWithLeanIX:nil offlineProvider:valueProvider];
    [builder onlineProvider:valueProvider];
    termsOfUseProvider = [builder build];
}

- (void)initActivityConsentProvider:(NSString*)activityId {
    TTPActivityProviderBuilder* builder = [[TTPActivityProviderBuilder alloc] initWithActivityId:activityId offlineProvider:[[ActivityConsentOfflineProvider alloc] init]];
    activityProvider = [builder build];
}

@end

