//
//  RNCountrySelectorManager.m
//  RNPrivacyComponent
//
//  Created by István Bege on 28/10/2018.
//  Copyright (C) 2018, Bosch Thermotechnik GmbH. All rights reserved.
//
#import <React/RCTViewManager.h>
#import <React/RCTLog.h>
#import <React/RCTUIManager.h>

#import "RNTTPCountrySelector.h"
#import "OfflineValueProvider.h"
#import "ProviderManager.h"
#import "Assembler.h"

@interface RNCountrySelectorManager : RCTViewManager <TTPCountrySelectorListener>
@end

@implementation RNCountrySelectorManager

RCT_EXPORT_MODULE()

RCT_EXPORT_VIEW_PROPERTY(onCountrySelected, RCTBubblingEventBlock)

- (UIView *)view
{
    return [[RNTTPCountrySelector alloc] init];
}

RCT_EXPORT_METHOD(loadCountries:(nonnull NSNumber *)reactTag)
{
    [self.bridge.uiManager addUIBlock:^(RCTUIManager *uiManager, NSDictionary<NSNumber *,RNTTPCountrySelector *> *viewRegistry) {
        RNTTPCountrySelector* view = viewRegistry[reactTag];
        if (![view isKindOfClass:[RNTTPCountrySelector class]]) {
            RCTLogError(@"Invalid view returned from registry, expecting RNTTPCountrySelectorView, got: %@", view);
        } else {
            ProviderManager* providerManager = [ProviderManager getInstance];
            [view loadCountriesWithProvider:[providerManager privacyProvider] supportedCountries:[providerManager supportedCountries] selectedCountry:[providerManager selectedCountry] delegate:self];
        }
    }];
}

# pragma mark TTPCountrySelectorListener
- (void) onCountrySelected:(TTPCountry *)country countrySelector:(RNTTPCountrySelector *)countrySelector
{
    countrySelector.onCountrySelected([Assembler createCountryDTO:country]);
}

@end
